import sqlite3



class Database():

    def __init__(
            self,
            db_file: str = "brm.db"
        ) -> None:
        self.db_file = db_file
        self.conn = sqlite3.connect(self.db_file)
        self.cursor = self.conn.cursor()

    def init_db(self):
        sql = '''
        CREATE TABLE brm_job (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            job_id INTGER NOT NULL,
            job_group_id INTGER NOT NULL,
            work_dir TEXT NOT NULL,
            job_hash TEXT NOT NULL
        )
        '''
        self.cursor.execute(sql)
        self.conn.commit()

    def insert_record(self, job_id, job_group_id, work_dir, job_hash):
        sql = f'''
        INSERT INTO brm_job (job_id, job_group_id, work_dir, job_hash) VALUES ({job_id}, {job_group_id}, {work_dir}, {job_hash})
        '''
        self.cursor.execute(sql)
        self.conn.commit()


    def get_record(self, job_id):
        sql = f'''
        SELECT * FROM brm_job WHERE job_id = {job_id}
        '''
        self.cursor.execute(sql)
        return self.cursor.fetchone()
    
    def get_all_record(self, job_id):
        sql = f'''
        SELECT * FROM brm_job
        '''
        self.cursor.execute(sql)
        return self.cursor.fetchall()
    
    def update_record(self, job_id, job_group_id, work_dir, job_hash):
        sql = f'''
        UPDATE brm_job SET job_group_id = {job_group_id}, work_dir = {work_dir}, job_hash = {job_hash} WHERE job_id = {job_id}
        '''
        self.cursor.execute(sql)
        self.conn.commit()

    def delete_record(self, job_id):
        sql = f'''
        DELETE FROM brm_job WHERE job_id = {job_id}
        '''
        self.cursor.execute(sql)
        self.conn.commit()


import requests
import os
import json
import urllib
import getpass
import time
import configparser
import re

class RequestInfoException(Exception):
    pass

class Client:
    def __init__(
            self, 
            api_version: str = 'v2',
            email: str = "",
            password: str = "",
            base_url_v1: str = 'https://bohrium.dp.tech',
            base_url_v2: str = "https://openapi.dp.tech",
            token: str = "",
            debug: bool = False,
            use_config_file: bool = False,
            config_file_location_v1: str = '~/.lebesgue_config.json',
            config_file_location_v2: str ='~/.brmconfig'
        ) -> None:
        if api_version == "v1":
            self.debug = debug
            self.debug = os.getenv('LBG_CLI_DEBUG_PRINT', debug)
            self.config = {}
            config_file_location_expand = os.path.expanduser(config_file_location_v1)
            file_data = {}
            self.token = ''
            self.user_id = None
            if use_config_file:
                if os.path.exists(config_file_location_expand):
                    with open(config_file_location_expand, 'r') as f:
                        file_data = json.loads(f.read())
                self.config['email'] = file_data.get('email', email)
                self.config['password'] = file_data.get('password', password)
                self.base_url = file_data.get('base_url', base_url_v1)
            else:
                self.config['email'] = email
                self.config['password'] = password
                self.base_url = base_url_v1
            if token is not None:
                self.token = token
            else:
                self._login()
        elif api_version == "v2":
            self.config_file_location_expand = os.path.expanduser(config_file_location_v2)
            if not os.path.exists(self.config_file_location_expand):
                print("Config File ~/.brmconfig not found! Now login to bohrium and generate it!")
                self.login()
                access_key_name = input("Please enter access_key name: ")
                self.generate_access_key(access_key_name)
            config = configparser.ConfigParser()
            config.read(self.config_file_location_expand)
            self.base_url = config.get('Credentials', 'baseUrl')
            self.access_key = config.get('Credentials', 'accessKey')
            self.params = {"accessKey": self.access_key}
            self.token = ""
            

    def post(self, url, host="", json=None, data=None, headers=None, params=None, stream=False, retry=5):
        return self._req('POST', url, host=host, json=json, data=data, headers=headers, params=params, stream=stream, retry=retry)

    def get(self, url, host="", json=None, headers=None, params=None, stream=False, retry=5):
        return self._req('GET', url, host=host, json=json, headers=headers, params=params, stream=stream, retry=retry)

    def _req(self, method, url, host="", json=None, data=None, headers=None, params=None, stream=False, retry=5):
        if host: #in ["https://bohrium.test.dp.tech", "https://tiefblue.test.dp.tech"]:
            url = urllib.parse.urljoin(host, url)
        else:
            url = urllib.parse.urljoin(self.base_url, url)
        
        # Set Headers
        if headers is None: headers = {}
       
        if self.token: headers['Authorization'] = f'Bearer {self.token}'
     

        # headers['bohr-client'] = f'utility:0.0.2'
        resp_code = None
        for i in range(retry):
            resp = None
            err = ""
            if method == 'GET':
                resp = requests.get(url=url, params=params, headers=headers, stream=stream)
            if method == 'POST':
                resp = requests.post(url=url, json=json, data=data, params=params, headers=headers, stream=stream)
            resp_code = resp.status_code
            if not resp.ok:
                try:
                    result = resp.json()
                    err = result.get("error")
                except:
                    pass
                time.sleep(0.1 * i)
                continue
            # print(resp.text)
            result = resp.json()
            if isinstance(result, str): return result
            if result.get('model', '') == 'gpt-35-turbo':
                return result['choices'][0]['message']['content']
            elif result['code'] == 0:
                return result.get('data', {})
            else:
                err = result.get("message") or result.get("error")
                break
        raise RequestInfoException(resp_code, url, err)

    def get_token(self):
        self.login()
        return self.token


    def login(self):
        email = input("Please enter Bohrium Account Email: ")
        password = getpass.getpass(prompt="Please enter password: ")
        post_data = {
            'username': email,
            'password': password
        }
        resp = requests.post('https://bohrium.dp.tech/account_gw/login', json=post_data).json().get("data", {})
        self.token = resp.get('token', '')
        if self.token: print("Login successfully!")
        else: print("Login failed!")

    def generate_access_key(self, name="default"):
        post_data = { "name": name }
        headers = { 'Authorization': f'Bearer {self.token}' }
        resp = requests.post(url="https://bohrium-api.dp.tech/bohrapi/v1/ak/add", json=post_data, headers=headers)
        print(resp)
        resp = resp.json().get("data", {})
        self.access_key = resp.get("accessKey", "")
        data = f"[Credentials]\nbaseUrl=https://openapi.dp.tech\naccessKey={self.access_key}"
        with open(self.config_file_location_expand, 'w') as f:
            f.write(data)
        return resp


    def chat(self, prompt, temperature=0):
        post_data = {
            "messages":[{"role":"user","content":f"{prompt}"}],
            "stream":False,
            "model":"gpt-3.5-turbo",
            "temperature":temperature,
            "presence_penalty":0
        }

        resp = self.post(f"/openapi/v1/chat/complete", json=post_data, params=self.params)
        return resp



from bohriumsdk.client import Client
from bohriumsdk.util import Util
class Node:
    def __init__(self, client):
        self.client = client
        

    def list_server(self, project_id):
        self.client.params["projectId"] = project_id
        self.client.params["device"] = "container"
        data = self.client.get('/openapi/v1/node/list', params=self.client.params)
        return data['items']

    def stop(self, machine_id, creator_id):
        data = self.client.post(f'/openapi/v1/node/stop/{machine_id}', data={"creatorId": creator_id}, params=self.client.params)
        return data

    def restart(self, machine_id):
        data = self.client.post(f'/openapi/v1/node/restart/{machine_id}', params=self.client.params)
        return data

    def delete(self, machine_id, creator_id):
        data = self.client.post(f'/openapi/v1/node/del/{machine_id}', data={"creatorId": creator_id}, params=self.client.params)
        return data

    def create(self, image_id, disk_size, memory, cpu, gpu, project_id, name=None):
        post_data = {
            'imageId': image_id,
            'diskSize': disk_size,
            'projectId': project_id,
            'memory': memory,
            'cpu': cpu,
            'gpu': gpu,
            'name': name
        }
        data = self.client.post(f'/openapi/v1/node/add', data=post_data, params=self.client.params)
        return data

    def to_dev_image(self, machine_id, image_name, comment=''):
        post_data = {
            'imageName': image_name,
            'nodeId': machine_id,
            'comment': comment
        }
        data = self.client.post(f'/openapi/v1/image/add', data=post_data, params=self.client.params)
        return data

    def print_node(
            self,
            project_id: int = 0 
        ) -> None:
        data = self.list_server(project_id)
        headers = ['nodeName','ip','nodePwd','cpu','memory','imageName', "cost",  'spec', 'createTime', 'diskSize', 'device']
        items = []
        for row in data:
            i = [row[k] for k in headers]
            items.append(i)
        Util().nice_print_table(headers=headers, items=items)




from bohriumsdk.client import Client
from bohriumsdk.util import Util

class Image(object):
    def __init__(
            self, 
            client: Client = None
        ) -> None:
        self.client = client

    def list_image_by_page(
            self, 
            project_id: int = 1, 
            kind=None, 
            page=1, 
            per_page=30
        ) -> None:
        params = {
            'page': page, 
            'pageSize': per_page, 
            'projectId': project_id, 
            "type": "private"
        }
        if kind is not None:
            params['kind'] = kind
        url = '/brm/v1/image/list'
        host = "https://bohrium.dp.tech"
        data = self.client.get(host=host, url=url, params=params)
        #print(data)
        return data
    
    def list_all_image(self, project_id, kind=None):
        project_list = []
        data = self.list_image_by_page(project_id=project_id, kind=kind)
        total = data['total']
        per_page = data['pageSize']
        page_number = 0
        while page_number * per_page < total:
            page_number = page_number + 1
            if page_number > 1:
                data = self.list_image_by_page(project_id=project_id, kind=kind, page=page_number, per_page=30)
            project_list.extend(data['items'])
        return project_list


    def delete(self, image_id):
        host = "https://bohrium.dp.tech"
        url = f'/brm/v1/image/del/{image_id}'
        data = self.client.get(host=host, url=url)
        
        return data

    def print_image(
            self,
            project_id: int = 0 
        ) -> None:
        data = self.list_all_image(project_id)
        headers = ['imageName','createTime', "creatorName",  'diskSize', 'projectRole', 'status']
        items = []
        for row in data:
            i = [row[k] for k in headers]
            items.append(i)
        Util().nice_print_table(headers=headers, items=items)

    

    



from client import Client
from image import Image
from job import Job
from storage import Storage
from project import Project
from util import Util
from node import Node
from database import Database
import os, sys

def test_image():
    c = Client()
    c.login()
    i = Image(c)
    res = i.list_image_by_page(project_id=154)
    #print(res["items"][0].keys())
    i.print_image(154)


def test_job():
    c = Client()
    # c.login()
    # print(c.token)
    # ak = c.generate_access_key("ak")
    # print(ak)
    j = Job(c)
    s = Storage(client = c)
    #data = j.detail()
    #print(j.log(10009242))
    data = j.create(project_id=154, name="test-env-job")
    
    # print(data)
    file_path = "/Users/dingzhaohan/Downloads/out.zip"
    file_name = os.path.basename(file_path)
    token = data["token"]
    print(data)
    #print(j.list_by_page(job_group_id=2107148))
    jid = data["jobId"]
    # j.get_job_token(jid)
    object_key = os.path.join(data["storePath"], file_name)
    res = s.upload_From_file_multi_part(object_key=object_key, file_path=file_path,  token=token, progress_bar=True)
    #res = s.upload_from_file(object_key=object_key, file_path=file_path, token=token)
    # s.download_from_file()

    #res = s.read(object_key=res["path"], token=token)
    #print(res)

    data = {
        "oss_path": object_key,
        'input_file_type': 3,
        'input_file_method': 1,
        "job_type": "container",
        "job_name": "test-name",
        "project_id": 154,
        "scass_type": "c2_m4_cpu",
        "command": "echo 1asdfasdfasdfasdfasdfasdfasdf11 >> test.result;",
        "log_files": ["test.result"],
        "platform": "ali",
        "image_address": "registry.dp.tech/dev/test/ubuntu:20.04-py3.10",
        "job_id": jid
    }
    j.insert(**data)
    #print(a["items"][0].keys())

def test_node():
    c = Client()
    node = Node(c)
    node.print_node(154)
    # data = node.list_server(154)
    # print(data)
    # headers = data[0].keys()
    # print(list(headers))
    # headers = ['nodeName','ip','nodePwd','cpu','memory','imageName', "cost",  'spec', 'createTime', 'diskSize', 'device']
    # items = []
    # for i in data:
    #     tmp_data = []
    #     # tmp_data = list(i.values())
    #     for k in headers:
    #         tmp_data.append(i[k])
    #     #print(tmp_data)
    #     items.append(tmp_data)
    # u = Util()
    # u.nice_print_table(headers=headers, items=items)

def test_project():
    c = Client()
    c.login()
    p = Project(client=c)
    p.print_project()

def test_storage():
    c = Client()
    j = Job(client=c)
    s = Storage(client = c)

    resp = j.create(project_id=154, name="upload_test")
    file_path = "/Users/dingzhaohan/Desktop/Bohrium_CALYPSO_example.zip"
    file_name = os.path.basename(file_path)
    token = resp["token"]
    object_key = os.path.join(resp["storePath"], file_name)

    res = s.upload_From_file_multi_part(file_path=file_path, object_key=object_key, token=token)
    #res = s.upload_from_file(object_key=object_key, file_path=file_path, token=token)
    print(res)
    # filename = "a.txt"
    # object_key = os.path.join(resp["storePath"], filename)
    # 
    # print(resp)
    # param = Parameter()
    # param.userMeta = {
    #         "a": "b",
    #         "ever":"17"
    #     }
    # param.contentType = "text/plain"
    # param.contentDisposition = f"attachment; filename={filename}"
    # param.filename = filename
    # res = s.write(object_key=object_key, data="1231231231231", token=token, parameter=param)
    # print(res)

    res = s.read(object_key=res["path"], token=token)
    print(res)

def test_base64():
    encode_data = "eyJhcHBLZXkiOiJib2hyLXRlc3QiLCJ0YWciOiJjbG91ZCIsInJlc291cmNlcyI6W3siUHJlZml4IjoiMTU3L2pvYi8yMDEzNDE0NS9pbnB1dC8iLCJBY3Rpb24iOjE1fV0sInByZWRlZmluZWRNZXRhIjpudWxsLCJleHAiOjE2ODYyNzQxOTl9"
    c = Client()
    s = Storage(c)
    decode = s.decode_base64(encode_data=encode_data)
    print(decode)

def test_db():
    db = Database()
    db.init_db()
    db.insert_record(1,1,"/home/test", "abcd")
    db.insert_record(2,1,"/home/test", "abcd")
    db.get_all_record()
    db.get_record(2)
    db.update_record(2,1,"/root/test", "aaaaaa")
    db.get_record(2)
    db.delete_record(2)
    db.get_all_record()


def test_chat():

    c = Client()
    prompt = "你好"
    res = c.chat(prompt)
    print(res)


if __name__ == "__main__":
    test_chat()
    # test_job()
    # test_base64()
    # test_db()



from bohriumsdk.client import Client
from bohriumsdk.util import Util
class Project(object):
    def __init__(
            self, 
            client: Client = None
        ) -> None:
        self.client = client

    def list_project_by_page(
            self,
            page: int = 1,
            per_page: int = 30
        ) -> None:
        host = "https://bohrium.dp.tech"
        # host = "https://openapi.dp.tech/"
        url = "/account/programs"
        params = {
            "page": page,
            "per_page": per_page
        }
        #self.client.token = "Bearer eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE2ODc2ODE2MTEsImlkZW50aXR5Ijp7Im9yZ0lkIjoxNTcsInVzZXJJZCI6MTU3fSwib3JpZ19pYXQiOjE2ODUwODk2MTF9.OGASZRsHqAKILDSYqyfd76yVktQ8L_oIHFFvWmY54yl7qFaSXXgx6-9_1KoeZG4S36tZFfk8isZaBPpTSTIX6JMkYqdLtxVBGogr4np3hz_ZV8a3set5uTst47Rwa_7Yk1v2iAU4FrghnKemnVu8e4uJSH2pClLgPQVZKF1CZd3AlG55fAvqnzY0-9uiKyBYmfuvUNM1ZlMJXHmFJ400NKZWkZj4iygJ03wGjKBBeXhyIZ4EyIbGrP7683JN2jOyw4wJJISAvKsv_gTNw_mRkdFsZuYx-u19II2UhqCAvMJtwjwgtIY4ReNfEoQPBUJWVyFBi7vaNbkwk-fghr2L0g"
        data = self.client.get(url=url, host=host, params=params)
        return data
    
    def list_all_project(self):
        host = "https://bohrium.dp.tech"
        url = "/brm/v1/project/list"
        params = {
            'page': 1,
            'pageSize': 10000,
            'queryType': 'all'
        }
        data = self.client.get(url=url, host=host, params=params)
        return data["items"]
    
    # 返回data盘大小
    def file_account(self, project_id):
        host = "https://bohrium.dp.tech"
        url = "/brm/v1/file/accounting"
        params = { "projectId": project_id }
        data = self.client.get(url=url, host=host, params=params)
        return data
    
    def print_project(self, env):
        data = self.list_all_project()
        headers = ['id', 'name', 'creatorEmail', 'jobCount', 'nodeCount', 'imageLimit', 'nodeLimit', 'storageLimit', 'storageUsed', 'costLimit', 'totalCost', 'userCost', 'userCostLimit']
        items = []
        for row in data:
            i = [row[k] for k in headers]
            items.append(i)
        Util().nice_print_table(headers=headers, items=items, env=env)




import rich
from rich import print
from rich.table import Table
from zipfile import ZipFile
import glob
import os

class Util(object):
    def __init__(self):
        pass

    def nice_print_table(
            self,
            headers: list = [],
            items: list = [],
            env: str = "terminal"
        ) -> None:
        
        table = Table(show_header=True, header_style="bold magenta", expand=True)
        for header in headers:
            table.add_column(header)
        
        for row in items:
            table.add_row(*[str(item) for item in row])
        # if env == "notebook":
        #     console = JupyterConsole()
        #     console.print(table)
        #     display(console) 
        # else:
        print(table)

    @classmethod
    def zip_file_list(cls, root_path, zip_filename, file_list=[]):
        out_zip_file = os.path.join(root_path, zip_filename)
        # print('debug: file_list', file_list)
        zip_obj = ZipFile(out_zip_file, "w")
        for f in file_list:
            matched_files = os.path.join(root_path, f)
            for ii in glob.glob(matched_files):
                # print('debug: matched_files:ii', ii)
                if os.path.isdir(ii):
                    arcname = os.path.relpath(ii, start=root_path)
                    zip_obj.write(ii, arcname)
                    for root, dirs, files in os.walk(ii):
                        for file in files:
                            filename = os.path.join(root, file)
                            arcname = os.path.relpath(filename, start=root_path)
                            # print('debug: filename:arcname:root_path', filename, arcname, root_path)
                            zip_obj.write(filename, arcname)
                else:
                    arcname = os.path.relpath(ii, start=root_path)
                    zip_obj.write(ii, arcname)
        zip_obj.close()
        return out_zip_file


    def unzip_file(zip_file, out_dir="./"):
        obj = ZipFile(zip_file, "r")
        for item in obj.namelist():
            obj.extract(item, out_dir)
            
if __name__ == '__main__':
    headers = [
      "Job ID",
      "Group Name",
      "Job Status",
      "Group ID",
      "Job Groups",
      "Project",
      "Duration",
      "Creation Time",
      "Cost",
      "Submission type"
    ]

    items = [
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
      [5578019, "dpgen_train_job", "Finished", 10362171, "dpgen_train_job", "dingzhaohan", "00:06:13", "2023-01-19 01:11:11", "¥0.46", "Command line"],
    ]
    util = Util()
    util.nice_print_table(headers=headers, items=items)

    print("asdf")


from setuptools import setup
import setuptools
setup(
    name="bohrium-sdk",
    version="0.1.10",
    author="dingzhaohan",
    author_email="dingzh@dp.tech",
    url="https://github.com/dingzhoahan",
    description="bohrium openapi python sdk",
    packages=setuptools.find_packages(),
    install_requires=[],
    python_requires='>=3.7',
    entry_points={
        'console_scripts': [
            'node_list=sdk.node:list_node',
            'node_delete=sdk.node:delete_node'
        ]
    }
)





from bohriumsdk.client import Client
from bohriumsdk.job import Job
import requests
import json
import base64
import os
from tqdm import tqdm

_DEFAULT_CHUNK_SIZE = 50 * 1024 * 1024
_DEFAULT_ITERATE_MAX_OBJECTS = 50
class Parameter(object):
    contentType: str
    contentEncoding: str
    contentLanguage: str
    contentDisposition: str
    cacheControl: str
    acl: str
    expires: str
    userMeta: dict
    predefinedMeta: str

class Storage:
    TIEFBLUE_HEADER_KEY = 'X-Storage-Param'

    def __init__(
            self,
            base_url: str = "https://openapi.dp.tech",
            client: Client = None,
        ) -> None:
        
        self.base_url = base_url
        self.host = "https://tiefblue.dp.tech"
        self.client = client
        pass
    
    def encode_base64(
            self, 
            parameter: dict = {}
        ) -> str:
        j = json.dumps(parameter)
        return base64.b64encode(j.encode()).decode()

    def write(
            self, 
            object_key: str = "", 
            token: str = "",
            data: str = "" , 
            parameter: dict = {}, 
            progress_bar: dict = {}
        ) -> dict:

        param = {
            "path": object_key,
            'option': parameter
        }

        if parameter:
            param["option"] = parameter.__dict__
        print(param)
        headers = {}
        headers[self.TIEFBLUE_HEADER_KEY] = self.encode_base64(param)
        headers['Authorization'] = "Bearer " + token

        # req = self.client.post(f"/api/upload/binary", data=body)
        url = f"/api/upload/binary"
        
        req = self.client.post(url=url, host=self.host, headers=headers, data=data)
        # req = requests.post("https://tiefblue.dp.tech/api/upload/binary", headers=headers, data=data)
        # self._raise_error(req)
        return req
    
    def read(
            self,
            object_key: str = "",
            token: str = "",
            ranges: str = ""
        ) -> None:

        url = f"/api/download/{object_key}"
        self.client.token = token
        res = self.client.get(url=url, host=self.host, stream=True)
        return res
    

    def upload_from_file(
            self,
            object_key: str = "",
            file_path: str = "",
            token: str = "",
            parameter: dict = None
        ) -> None:
        if not os.path.exists(file_path):
            raise FileNotFoundError
        if os.path.isdir(file_path):
            raise IsADirectoryError
        _, disposition = os.path.split(file_path)
        if parameter is None:
            parameter = Parameter()
        parameter.contentDisposition = f'attachment; filename="{disposition}"'
        with open(file_path, 'rb') as fp:
            res = self.write(object_key=object_key, data=fp.read(), token=token, parameter=parameter)
            return res
    
    

    def init_upload_by_part(self, object_key: str, parameter=None):
        data = {
            'path': object_key
        }
        if parameter is not None:
            data['option'] = parameter.__dict__

        url = f"/api/upload/multipart/init"
        resp = self.client.post(url=url, host=self.host, data=data)
        return resp
    
    def upload_by_part(self, object_key: str, initial_key: str, chunk_size: int, number: int, body):
        param = {
            'initialKey': initial_key,
            'number': number,
            'partSize': chunk_size,
            'objectKey': object_key
        }
        headers = {}
        headers[self.TIEFBLUE_HEADER_KEY] = self._dump_parameter(param)
        url = f"/api/upload/multipart/upload"
        resp = self.client.post(url=url, host=self.host, data=body, headers=headers)
        return resp


    def complete_upload_by_part(self, object_key, initial_key, part_string):
        data = {
            'path': object_key,
            'initialKey': initial_key,
            'partString': part_string
        }

        url = f"/api/upload/multipart/complete"
        resp = self.client.post(url=url, host=self.host, data=data)
        return resp
    
    def upload_From_file_multi_part(
            self,
            object_key: str,
            file_path: str,
            token: str,
            chunk_size: int = _DEFAULT_CHUNK_SIZE,
            parameter = None,
            progress_bar = False,
            need_parse = False
        ) -> None:
        if not os.path.exists(file_path):
            raise FileNotFoundError
        if os.path.isdir(file_path):
            raise IsADirectoryError
        if need_parse:
            _, _, object_key = self._parse_ap_name_and_tag(object_key)
        size = os.path.getsize(file_path)
        _, disposition = os.path.split(file_path)
        if parameter is None:
            parameter = Parameter()
        parameter.contentDisposition = f'attachment; filename="{disposition}"'
        bar_format = "{l_bar}{bar}| {n:.02f}/{total:.02f} %  [{elapsed}<{remaining}, {rate_fmt}{postfix}]"
        with open(file_path, 'r') as f:
            pbar = tqdm(total=100, desc=f"Uploading {disposition}", smoothing=0.01, bar_format=bar_format,
                        disable=not progress_bar)
            f.seek(0)
            if size < _DEFAULT_CHUNK_SIZE * 2:
                print(object_key)
                self.write(object_key=object_key, token=token, data=f.buffer, parameter=parameter)
                pbar.update(100)
                pbar.close()
                return
            chunks = split_size_by_part_size(size, chunk_size)
            initial_key = self.init_upload_by_part(object_key, parameter).get('initialKey')
            part_string = []
            for c in chunks:
                f.seek(c.Offset)
                num_to_upload = min(chunk_size, size - c.Offset)
                part_string.append(self.upload_by_part(object_key, initial_key, chunk_size=c.Size, number=c.Number,
                                                       body=f.buffer.read(c.Size)).get('partString'))
                percent = num_to_upload * 100 / (size + 1)
                pbar.update(percent)
            pbar.close()
            return self.complete_upload_by_part(object_key, initial_key, part_string)

    def download_from_file(self):

        data = self.client.get()


    def _dump_parameter(self, parameter):
        j = json.dumps(parameter)
        return base64.b64encode(j.encode()).decode()
    
    def decode_base64(self, encode_data):
        data = json.loads(base64.b64decode(encode_data).decode('utf-8'))
        return data

    def _parse_ap_name_and_tag(self, input_path: str):
        l = input_path.split('/')
        if len(l) < 3:
            return "", "". l
        return l[0], l[1], "/".join(l[2:])



class Chunk:
    Number: int
    Offset: int
    Size: int
   
def split_size_by_part_size(total_size: int, chunk_size: int):
    if chunk_size < _DEFAULT_CHUNK_SIZE:
        chunk_size = _DEFAULT_CHUNK_SIZE
    chunk_number = int(total_size / chunk_size)
    if chunk_number >= 10000:
        raise TooManyChunk
    chunks = []
    for i in range(chunk_number):
        c = Chunk()
        c.Number = i + 1
        c.Offset = i * chunk_size
        c.Size = chunk_size
        chunks.append(c)

    if total_size % chunk_size > 0:
        c = Chunk()
        c.Number = len(chunks) + 1
        c.Offset = len(chunks) * chunk_size
        c.Size = total_size % chunk_size
        chunks.append(c)
    return chunks


def partial_with_start_from(start_bytes):
    return f'bytes={start_bytes}-'


def partial_with_end_from(end_bytes):
    return f'bytes=-{end_bytes}'


def partial_with_range(start_bytes, end_bytes):
    return f'bytes={start_bytes}-{end_bytes}'


TooManyChunk = Exception("too many chunks, please consider increase your chunk size")



import requests
from bohriumsdk.client import Client
import humps

class Job:
    def __init__(
            self, 
            client: Client = None
        ) -> None:
        self.client = client

    def list_by_page(self, job_group_id, status=None, page=1, per_page=50):
        params = {
            'groupId': job_group_id,
            'page': page,
            'pageSize': per_page
        }
        if status:
            params['status'] = status
        # print(self.client.access_key)
        params['accessKey'] = self.client.access_key
        data = self.client.get(f'/openapi/v1/job/list', params=params)
        return data
    

    def list_by_number(self, job_group_id, number, status=None):
        if status is None:
            status = []
        per_page = 50
        job_list = []
        data = self.list_by_page(job_group_id, page=1, per_page=per_page, status=status)
        total = data.get("total")
        per_page = data.get("pageSize")
        page_number = 0
        while page_number * per_page < total:
            page_number = page_number + 1
            if page_number > 1:
                data = self.list_by_page(job_group_id, page_number, per_page, status)
            for each in data.get("items"):
                job_list.append(each)
                if number != -1 and len(job_list) >= number:
                    return job_list
        return job_list
    
    def delete(self, job_id):
        data = self.client.post(f"/openapi/v1/job/del/{job_id}", params=self.client.params)
        return data
    
    def terminate(self, job_id):
        data = self.client.post(f'/openapi/v1/job/terminate/{job_id}', params=self.client.params)
        return data
    
    def kill(self, job_id):
        data = self.client.post(f'/openapi/v1/job/kill/{job_id}', params=self.client.params)
        return data
    
    def log(self, job_id):
        data = self.client.get(f'/openapi/v1/job/{job_id}/log', params=self.client.params)
        return data

    def insert(self, **kwargs):
        must_fill = ['job_type', 'oss_path', 'project_id', 'scass_type', 'command', 'platform', 'image_address', 'job_id']
        # must_fill = ['job_type', 'oss_path', 'project_id', 'scass_type', 'command', 'platform', 'image_name']
        for each in must_fill:
            if each not in kwargs:
                raise ValueError(f'{each} is required when submitting job')
        camel_data = {humps.camelize(k): v for k, v in kwargs.items()}
        if not isinstance(camel_data['ossPath'], list):
            camel_data['ossPath'] = [camel_data['ossPath']]
        if 'logFile' in camel_data:
            camel_data['logFiles'] = camel_data['logFile']
        if 'logFiles' in camel_data and not isinstance(camel_data['logFiles'], list):
            camel_data['logFiles'] = [camel_data['logFiles']]
        #if self.client.debug:
        print(camel_data)
        data = self.client.post(f"/openapi/v2/job/add", json=camel_data, params=self.client.params)
        return data


    def detail(self, job_id):
        data = self.client.get(f'/openapi/v1/job/{job_id}', params=self.client.params)
        return data
    
    def create(self, project_id, name='', group_id=0):
        data = {
            'projectId': project_id
        }
        if name:
            data['name'] = name
        if group_id:
            data['groupId'] = group_id
        try:
            data = self.client.post(f'/openapi/v1/job/create', json=data, params=self.client.params)
        except Exception as e:
            raise e
        return data
    
    def create_job_group(self, project_id, job_group_name):
        data = {
            "name": job_group_name,
            "projectId": project_id
        }
        try:
            resp = self.client.post(f"/openapi/v1/job_group/add", json=data, params=self.client.params).json()
        except Exception as e:
            raise e
        return resp["data"]
    
    def get_job_token(self, job_id):
        url = f"/openapi/v1/job/{job_id}/input/token"

        data = self.client.get(url=url, params=self.client.params)
        print(data)


